@@ -71,6 +71,10 @@
 #define BUFSIZE	        16384
 #endif
 
+#ifndef BUFSIZE_SMALL
+#define BUFSIZE_SMALL   1024
+#endif
+
 // number of per-thread emergency buffers for low-memory conditions
 #ifndef RESERVED_BUFS
 #define RESERVED_BUFS   4
@@ -106,6 +110,11 @@
 #define LINESIZE	2048
 #endif
 
+// maximum size of a configuration file that could be loaded in memory via
+// /dev/sdtin. This is needed to prevent from loading extremely large files
+// via standard input.
+#define MAX_CFG_SIZE	10485760
+
 // max # args on a configuration line
 #define MAX_LINE_ARGS   64
 
@@ -313,11 +322,9 @@
 #define DEFAULT_MAXFD 1048576
 #endif
 
-/* Define a maxconn which will be used in the master process once it re-exec to
- * the MODE_MWORKER_WAIT and won't change when SYSTEM_MAXCONN is set.
- *
- * 100 must be enough for the master since it only does communication between
- * the master and the workers, and the master CLI.
+/* Define a maxconn which will be used in the master process and won't change
+ * when SYSTEM_MAXCONN is set. 100 must be enough for the master since it only
+ * does communication between the master and the workers, and the master CLI.
  */
 #ifndef MASTER_MAXCONN
 #define MASTER_MAXCONN 100
