@@ -35,6 +35,7 @@ extern int  unstoppable_jobs;   /* # of active jobs that can't be stopped during
 extern int  active_peers;       /* # of active peers (connection attempts and successes) */
 extern int  connected_peers;    /* # of really connected peers */
 extern int nb_oldpids;          /* contains the number of old pids found */
+extern int oldpids_sig;         /* signal to sent in order to stop the previous (old) process */
 extern const int zero;
 extern const int one;
 extern const struct linger nolinger;
@@ -45,11 +46,14 @@ extern char *localpeer;
 extern unsigned int warned;     /* bitfield of a few warnings to emit just once */
 extern struct list proc_list; /* list of process in mworker mode */
 extern int master; /* 1 if in master, 0 otherwise */
-extern unsigned int rlim_fd_cur_at_boot;
-extern unsigned int rlim_fd_max_at_boot;
 extern int atexit_flag;
 extern unsigned char boot_seed[20];  // per-boot random seed (160 bits initially)
 extern THREAD_LOCAL struct buffer trash;
+extern char **init_env;
+extern char *progname;
+extern char **old_argv;
+extern const char *old_unixsocket;
+extern int daemon_fd[2];
 
 struct proxy;
 struct server;
@@ -57,6 +61,7 @@ int main(int argc, char **argv);
 void deinit(void);
 __attribute__((noreturn)) void deinit_and_exit(int);
 void run_poll_loop(void);
+void *run_thread_poll_loop(void *data); /* takes the thread config in argument or NULL for any thread */
 int tell_old_pids(int sig);
 int delete_oldpid(int pid);
 void hap_register_build_opts(const char *str, int must_free);
@@ -64,9 +69,9 @@ void hap_register_feature(const char *name);
 int split_version(const char *version, unsigned int *value);
 int compare_current_version(const char *version);
 void display_version();
+int handle_pidfile(void);
 
 void mworker_accept_wrapper(int fd);
-void mworker_reload(int hardreload);
 
 /* to be used with warned and WARN_* */
 static inline int already_warned(unsigned int warning)
