@@ -32,7 +32,7 @@
 
 #include <haproxy/chunk.h>
 #include <haproxy/dynbuf.h>
-#include <haproxy/ncbmbuf.h>
+#include <haproxy/ncbuf.h>
 #include <haproxy/net_helper.h>
 #include <haproxy/openssl-compat.h>
 #include <haproxy/ticks.h>
@@ -45,6 +45,7 @@
 #include <haproxy/quic_enc.h>
 #include <haproxy/quic_frame.h>
 #include <haproxy/quic_loss.h>
+#include <haproxy/quic_pacing.h>
 #include <haproxy/quic_rx.h>
 #include <haproxy/mux_quic.h>
 
@@ -60,10 +61,6 @@ void quic_free_arngs(struct quic_conn *qc, struct quic_arngs *arngs);
 struct quic_cstream *quic_cstream_new(struct quic_conn *qc);
 struct task *quic_conn_app_io_cb(struct task *t, void *context, unsigned int state);
 
-struct quic_connection_id *new_quic_cid(struct eb_root *root,
-                                        struct quic_conn *qc,
-                                        const struct quic_cid *orig,
-                                        const struct sockaddr_storage *addr);
 void quic_conn_closed_err_count_inc(struct quic_conn *qc, struct quic_frame *frm);
 int qc_h3_request_reject(struct quic_conn *qc, uint64_t id);
 struct quic_conn *qc_new_conn(const struct quic_version *qv, int ipv4,
@@ -130,40 +127,36 @@ static inline void quic_conn_mv_cids_to_cc_conn(struct quic_conn_closed *cc_conn
 
 }
 
-/* Allocate the underlying required memory for <ncbuf> non-contiguous buffer.
- * Does nothing if buffer is already allocated.
- *
- * Returns the buffer instance or NULL on allocation failure.
- */
-static inline struct ncbmbuf *quic_get_ncbuf(struct ncbmbuf *ncbuf)
+/* Allocate the underlying required memory for <ncbuf> non-contiguous buffer */
+static inline struct ncbuf *quic_get_ncbuf(struct ncbuf *ncbuf)
 {
 	struct buffer buf = BUF_NULL;
 
-	if (!ncbmb_is_null(ncbuf))
+	if (!ncb_is_null(ncbuf))
 		return ncbuf;
 
-	if (!b_alloc(&buf, DB_MUX_RX))
-		return NULL;
+	b_alloc(&buf, DB_MUX_RX);
+	BUG_ON(b_is_null(&buf));
 
-	*ncbuf = ncbmb_make(buf.area, buf.size, 0);
-	ncbmb_init(ncbuf, 0);
+	*ncbuf = ncb_make(buf.area, buf.size, 0);
+	ncb_init(ncbuf, 0);
 
 	return ncbuf;
 }
 
 /* Release the underlying memory use by <ncbuf> non-contiguous buffer */
-static inline void quic_free_ncbuf(struct ncbmbuf *ncbuf)
+static inline void quic_free_ncbuf(struct ncbuf *ncbuf)
 {
 	struct buffer buf;
 
-	if (ncbmb_is_null(ncbuf))
+	if (ncb_is_null(ncbuf))
 		return;
 
 	buf = b_make(ncbuf->area, ncbuf->size, 0, 0);
 	b_free(&buf);
 	offer_buffers(NULL, 1);
 
-	*ncbuf = NCBMBUF_NULL;
+	*ncbuf = NCBUF_NULL;
 }
 
 void chunk_frm_appendf(struct buffer *buf, const struct quic_frame *frm);
@@ -171,12 +164,6 @@ void quic_set_connection_close(struct quic_conn *qc, const struct quic_err err);
 void quic_set_tls_alert(struct quic_conn *qc, int alert);
 int quic_set_app_ops(struct quic_conn *qc, const unsigned char *alpn, size_t alpn_len);
 int qc_check_dcid(struct quic_conn *qc, unsigned char *dcid, size_t dcid_len);
-struct quic_cid quic_derive_cid(const struct quic_cid *orig,
-                                const struct sockaddr_storage *addr);
-int quic_get_cid_tid(const unsigned char *cid, size_t cid_len,
-                     const struct sockaddr_storage *cli_addr,
-                     unsigned char *buf, size_t buf_len);
-int qc_send_mux(struct quic_conn *qc, struct list *frms);
 
 void qc_notify_err(struct quic_conn *qc);
 int qc_notify_send(struct quic_conn *qc);
