@@ -206,7 +206,6 @@ enum sc_flags {
 
 	SC_FL_EOS           = 0x00040000,  /* End of stream was reached (from down side to up side) */
 	SC_FL_HAVE_BUFF     = 0x00080000,  /* A buffer is ready, flag will be cleared once allocated */
-	SC_FL_NO_FASTFWD    = 0x00100000,  /* disable data fast-forwarding */
 };
 
 /* This function is used to report flags in debugging tools. Please reflect
@@ -224,7 +223,7 @@ static forceinline char *sc_show_flags(char *buf, size_t len, const char *delim,
 	_(SC_FL_NEED_BUFF, _(SC_FL_NEED_ROOM,
         _(SC_FL_RCV_ONCE, _(SC_FL_SND_ASAP, _(SC_FL_SND_NEVERWAIT, _(SC_FL_SND_EXP_MORE,
 	_(SC_FL_ABRT_WANTED, _(SC_FL_SHUT_WANTED, _(SC_FL_ABRT_DONE, _(SC_FL_SHUT_DONE,
-	_(SC_FL_EOS, _(SC_FL_HAVE_BUFF, _(SC_FL_NO_FASTFWD)))))))))))))))))))));
+	_(SC_FL_EOS, _(SC_FL_HAVE_BUFF))))))))))))))))))));
 	/* epilogue */
 	_(~0U);
 	return buf;
@@ -277,10 +276,11 @@ struct stconn;
 #define SE_ABRT_SRC_MASK  0x0000001f
 
 #define SE_ABRT_SRC_MUX_PT    0x01 /* Code set by the PT mux */
-#define SE_ABRT_SRC_MUX_H1    0x02 /* Code set bu the H1 mux */
-#define SE_ABRT_SRC_MUX_H2    0x03 /* Code set bu the H2 mux */
-#define SE_ABRT_SRC_MUX_QUIC  0x04 /* Code set bu the QUIC/H3 mux */
-#define SE_ABRT_SRC_MUX_FCGI  0x05 /* Code set bu the FCGI mux */
+#define SE_ABRT_SRC_MUX_H1    0x02 /* Code set by the H1 mux */
+#define SE_ABRT_SRC_MUX_H2    0x03 /* Code set by the H2 mux */
+#define SE_ABRT_SRC_MUX_QUIC  0x04 /* Code set by the QUIC/H3 mux */
+#define SE_ABRT_SRC_MUX_FCGI  0x05 /* Code set by the FCGI mux */
+#define SE_ABRT_SRC_MUX_SPOP  0x06 /* Code set by the SPOP mux */
 
 struct se_abort_info {
 	uint32_t info;
