@@ -140,7 +140,7 @@ static inline void stream_store_counters(struct stream *s)
 			stktable_touch_local(s->stkctr[i].table, ts, 0);
 		}
 		stkctr_set_entry(&s->stkctr[i], NULL);
-		stksess_kill_if_expired(s->stkctr[i].table, ts, 1);
+		stksess_kill_if_expired(s->stkctr[i].table, ts);
 	}
 }
 
@@ -182,7 +182,7 @@ static inline void stream_stop_content_counters(struct stream *s)
 			stktable_touch_local(s->stkctr[i].table, ts, 0);
 		}
 		stkctr_set_entry(&s->stkctr[i], NULL);
-		stksess_kill_if_expired(s->stkctr[i].table, ts, 1);
+		stksess_kill_if_expired(s->stkctr[i].table, ts);
 	}
 }
 
@@ -350,7 +350,7 @@ static inline void stream_choose_redispatch(struct stream *s)
 	     (((s->be->redispatch_after > 0) &&
 	       (s->conn_retries % s->be->redispatch_after == 0)) ||
 	      ((s->be->redispatch_after < 0) &&
-	       (s->conn_retries % (s->be->conn_retries + 1 + s->be->redispatch_after) == 0))) ||
+	       (s->conn_retries % (s->max_retries + 1 + s->be->redispatch_after) == 0))) ||
 	     (!(s->flags & SF_DIRECT) && s->be->srv_act > 1 &&
 	      ((s->be->lbprm.algo & BE_LB_KIND) != BE_LB_KIND_HI)))) {
 		sess_change_server(s, NULL);
