@@ -934,6 +934,7 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 	const char *str;
 	int max_lines;
 	int i, j, max;
+	int dumped;
 
 	chunk_reset(&trash);
 
@@ -997,11 +998,17 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 	if (!max_lines)
 		max_lines = SCHED_ACT_HASH_BUCKETS;
 
+	dumped = 0;
 	for (i = ctx->linenum; i < max_lines; i++) {
 		if (!tmp_activity[i].calls)
 			continue; // skip aggregated or empty entries
 
 		ctx->linenum = i;
+
+		/* resolve_sym_name() may be slow, better dump a few entries at a time */
+		if (dumped >= 10)
+			return 0;
+
 		chunk_reset(name_buffer);
 		caller = HA_ATOMIC_LOAD(&tmp_activity[i].caller);
 
@@ -1034,6 +1041,7 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 			/* failed, try again */
 			return 0;
 		}
+		dumped++;
 	}
 
 	if (applet_putchk(appctx, &trash) == -1) {
@@ -1069,12 +1077,18 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 	if (!max_lines)
 		max_lines = MEMPROF_HASH_BUCKETS + 1;
 
+	dumped = 0;
 	for (i = ctx->linenum; i < max_lines; i++) {
 		struct memprof_stats *entry = &tmp_memstats[i];
 
 		ctx->linenum = i;
 		if (!entry->alloc_calls && !entry->free_calls)
 			continue;
+
+		/* resolve_sym_name() may be slow, better dump a few entries at a time */
+		if (dumped >= 10)
+			return 0;
+
 		chunk_appendf(&trash, "%11llu %11llu %14llu %14llu| %16p ",
 			      entry->alloc_calls, entry->free_calls,
 			      entry->alloc_tot, entry->free_tot,
@@ -1085,9 +1099,7 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 		else
 			chunk_appendf(&trash, "[other]");
 
-		if ((tmp_memstats[i].method != MEMPROF_METH_P_ALLOC) &&
-		    (tmp_memstats[i].method != MEMPROF_METH_MALLOC) &&
-		    (tmp_memstats[i].method != MEMPROF_METH_CALLOC)) {
+		if (((1UL << tmp_memstats[i].method) & MEMPROF_FREE_MASK) || !entry->alloc_calls) {
 			chunk_appendf(&trash," %s(%lld)", memprof_methods[entry->method],
 				(long long)(entry->alloc_tot - entry->free_tot) / (long long)(entry->alloc_calls + entry->free_calls));
 		} else
@@ -1109,6 +1121,8 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 
 		if (applet_putchk(appctx, &trash) == -1)
 			return 0;
+
+		dumped++;
 	}
 
 	if (applet_putchk(appctx, &trash) == -1)
@@ -1118,9 +1132,7 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 	for (i = 0; i < max_lines; i++) {
 		tot_alloc_calls += tmp_memstats[i].alloc_calls;
 		tot_alloc_bytes += tmp_memstats[i].alloc_tot;
-		if ((tmp_memstats[i].method != MEMPROF_METH_P_ALLOC) &&
-		    (tmp_memstats[i].method != MEMPROF_METH_MALLOC) &&
-		    (tmp_memstats[i].method != MEMPROF_METH_CALLOC)) {
+		if ((1UL << tmp_memstats[i].method) & MEMPROF_FREE_MASK) {
 			tot_free_calls  += tmp_memstats[i].free_calls;
 			tot_free_bytes  += tmp_memstats[i].free_tot;
 		}
@@ -1134,29 +1146,19 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 	 */
 	for (i = max = 0; i < max_lines; i++) {
 		struct memprof_stats *entry = &tmp_memstats[i];
-		char *p;
 
 		if (!entry->alloc_calls && !entry->free_calls)
 			continue;
 
 		chunk_reset(name_buffer);
 		if (!entry->caller)
-			chunk_printf(name_buffer, "other:");
+			chunk_printf(name_buffer, "other");
 		else
-			resolve_sym_name(name_buffer, "", entry->caller);
-
-		/* figure the DSO name (before colon) otherwise "*program*" */
-		p = strchr(name_buffer->area, ':');
-		if (p) {
-			*p = 0;
-			p = name_buffer->area;
-		}
-		else
-			p = "*program*";
+			resolve_dso_name(name_buffer, "", entry->caller);
 
 		/* look it up among known names (0..max) */
 		for (j = 0; j < max; j++) {
-			if (tmp_memstats[j].info && strcmp(p, tmp_memstats[j].info) == 0)
+			if (tmp_memstats[j].info && strcmp(name_buffer->area, tmp_memstats[j].info) == 0)
 				break;
 		}
 
@@ -1169,12 +1171,10 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 			if (j != i) // set max to keep min caller's address
 				tmp_memstats[j].caller = (void*)-1;
 
-			tmp_memstats[j].info = strdup(p);   // may fail, but checked when used
+			tmp_memstats[j].info = strdup(name_buffer->area);   // may fail, but checked when used
 			tmp_memstats[j].alloc_calls = entry->alloc_calls;
 			tmp_memstats[j].alloc_tot   = entry->alloc_tot;
-			if ((entry->method != MEMPROF_METH_P_ALLOC) &&
-			    (entry->method != MEMPROF_METH_MALLOC) &&
-			    (entry->method != MEMPROF_METH_CALLOC)) {
+			if ((1UL << entry->method) & MEMPROF_FREE_MASK) {
 				tmp_memstats[j].free_calls  = entry->free_calls;
 				tmp_memstats[j].free_tot    = entry->free_tot;
 			} else {
@@ -1184,9 +1184,7 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 		} else {
 			tmp_memstats[j].alloc_calls += entry->alloc_calls;
 			tmp_memstats[j].alloc_tot += entry->alloc_tot;
-			if ((entry->method != MEMPROF_METH_P_ALLOC) &&
-			    (entry->method != MEMPROF_METH_MALLOC) &&
-			    (entry->method != MEMPROF_METH_CALLOC)) {
+			if ((1UL << entry->method) & MEMPROF_FREE_MASK) {
 				tmp_memstats[j].free_calls  += entry->free_calls;
 				tmp_memstats[j].free_tot  += entry->free_tot;
 			}
@@ -1211,7 +1209,7 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 		chunk_appendf(&trash, "%11llu %11llu %14llu %14llu| %16p DSO:%s;",
 			      entry->alloc_calls, entry->free_calls,
 			      entry->alloc_tot, entry->free_tot,
-			      entry->caller, entry->info ? (const char*)entry->info : "other");
+			      entry->caller == (void*)-1 ? 0 : entry->caller, entry->info ? (const char*)entry->info : "other");
 
 		if (entry->alloc_tot != entry->free_tot)
 			chunk_appendf(&trash, " delta_calls=%lld; delta_bytes=%lld",
@@ -1329,7 +1327,7 @@ static int cli_io_handler_show_tasks(struct appctx *appctx)
 			t = eb32_entry(rqnode, struct task, rq);
 			entry = sched_activity_entry(tmp_activity, t->process, NULL);
 			if (t->wake_date) {
-				lat = (uint32_t)now_ns - t->wake_date;
+				lat = now_ns - t->wake_date;
 				if ((int64_t)lat > 0)
 					entry->lat_time += lat;
 			}
@@ -1346,7 +1344,7 @@ static int cli_io_handler_show_tasks(struct appctx *appctx)
 			t = eb32_entry(rqnode, struct task, rq);
 			entry = sched_activity_entry(tmp_activity, t->process, NULL);
 			if (t->wake_date) {
-				lat = (uint32_t)now_ns - t->wake_date;
+				lat = now_ns - t->wake_date;
 				if ((int64_t)lat > 0)
 					entry->lat_time += lat;
 			}
@@ -1359,7 +1357,7 @@ static int cli_io_handler_show_tasks(struct appctx *appctx)
 			t = (const struct task *)tl;
 			entry = sched_activity_entry(tmp_activity, t->process, NULL);
 			if (!TASK_IS_TASKLET(t) && t->wake_date) {
-				lat = (uint32_t)now_ns - t->wake_date;
+				lat = now_ns - t->wake_date;
 				if ((int64_t)lat > 0)
 					entry->lat_time += lat;
 			}
@@ -1372,7 +1370,7 @@ static int cli_io_handler_show_tasks(struct appctx *appctx)
 				t = (const struct task *)tl;
 				entry = sched_activity_entry(tmp_activity, t->process, NULL);
 				if (!TASK_IS_TASKLET(t) && t->wake_date) {
-					lat = (uint32_t)now_ns - t->wake_date;
+					lat = now_ns - t->wake_date;
 					if ((int64_t)lat > 0)
 						entry->lat_time += lat;
 				}
