@@ -148,10 +148,9 @@ struct h3c {
 
 DECLARE_STATIC_POOL(pool_head_h3c, "h3c", sizeof(struct h3c));
 
-#define H3_SF_UNI_INIT     0x00000001  /* stream type not parsed for unidirectional stream */
-#define H3_SF_UNI_NO_H3    0x00000002  /* unidirectional stream does not carry H3 frames */
-#define H3_SF_HAVE_CLEN    0x00000004  /* content-length header is present */
-#define H3_SF_SENT_INTERIM 0x00000008  /* last response sent is 1xx interim. Used on FE side only. */
+#define H3_SF_UNI_INIT  0x00000001  /* stream type not parsed for unidirectional stream */
+#define H3_SF_UNI_NO_H3 0x00000002  /* unidirectional stream does not carry H3 frames */
+#define H3_SF_HAVE_CLEN 0x00000004  /* content-length header is present */
 
 struct h3s {
 	struct h3c *h3c;
@@ -654,7 +653,7 @@ static ssize_t h3_headers_to_htx(struct qcs *qcs, const struct buffer *buf,
 
 			if (!relaxed) {
 				/* we need to reject any control chars or '#' from the path,
-				 * unless option accept-invalid-http-request is set.
+				 * unless option accept-unsafe-violations-in-http-request is set.
 				 */
 				ctl = ist_find_range(list[hdr_idx].v, 0, '#');
 				if (unlikely(ctl) && http_path_has_forbidden_char(list[hdr_idx].v, ctl)) {
@@ -678,12 +677,7 @@ static ssize_t h3_headers_to_htx(struct qcs *qcs, const struct buffer *buf,
 				goto out;
 			}
 
-			flags |= HTX_SL_F_HAS_SCHM;
-			if (isteqi(list[hdr_idx].v, ist("http")))
-				flags |= HTX_SL_F_SCHM_HTTP;
-			else if (isteqi(list[hdr_idx].v, ist("https")))
-				flags |= HTX_SL_F_SCHM_HTTPS;
-			else if (!http_validate_scheme(list[hdr_idx].v)) {
+			if (!http_validate_scheme(list[hdr_idx].v)) {
 				TRACE_ERROR("invalid scheme pseudo-header", H3_EV_RX_FRAME|H3_EV_RX_HDR, qcs->qcc->conn, qcs);
 				h3s->err = H3_ERR_MESSAGE_ERROR;
 				qcc_report_glitch(h3c->qcc, 1);
@@ -735,11 +729,6 @@ static ssize_t h3_headers_to_htx(struct qcs *qcs, const struct buffer *buf,
 			goto out;
 		}
 	}
-	else {
-		h3s->err = H3_ERR_REQUEST_REJECTED;
-		len = -1;
-		goto out;
-	}
 
 	flags |= HTX_SL_F_VER_11;
 	flags |= HTX_SL_F_XFER_LEN;
@@ -763,21 +752,6 @@ static ssize_t h3_headers_to_htx(struct qcs *qcs, const struct buffer *buf,
 		goto out;
 	}
 
-	/* We're going to concatenate :authority with :path to form a URI. Some
-	 * characters must absolutely be avoided in :authority to make sure not
-	 * to result in a broken concatenation. See the following links for a
-	 * discussion on this topic:
-	 *   https://github.com/httpwg/http2-spec/pull/936
-	 *   https://github.com/haproxy/haproxy/issues/2941
-	 */
-	if (http_authority_has_forbidden_char(authority)) {
-		TRACE_ERROR("invalid character in authority", H3_EV_RX_FRAME|H3_EV_RX_HDR, qcs->qcc->conn, qcs);
-		h3s->err = H3_ERR_MESSAGE_ERROR;
-		qcc_report_glitch(h3c->qcc, 1);
-		len = -1;
-		goto out;
-	}
-
 	/* Ensure that final URI does not contains LWS nor CTL characters. */
 	for (i = 0; i < path.len; i++) {
 		unsigned char c = istptr(path)[i];
@@ -823,7 +797,7 @@ static ssize_t h3_headers_to_htx(struct qcs *qcs, const struct buffer *buf,
 
 		for (i = 0; i < list[hdr_idx].n.len; ++i) {
 			const char c = list[hdr_idx].n.ptr[i];
-			if ((uint8_t)(c - 'A') <= 'Z' - 'A' || !HTTP_IS_TOKEN(c)) {
+			if ((uint8_t)(c - 'A') < 'Z' - 'A' || !HTTP_IS_TOKEN(c)) {
 				TRACE_ERROR("invalid characters in field name", H3_EV_RX_FRAME|H3_EV_RX_HDR, qcs->qcc->conn, qcs);
 				h3s->err = H3_ERR_MESSAGE_ERROR;
 				qcc_report_glitch(h3c->qcc, 1);
@@ -854,21 +828,12 @@ static ssize_t h3_headers_to_htx(struct qcs *qcs, const struct buffer *buf,
 		}
 
 		if (isteq(list[hdr_idx].n, ist("host"))) {
-			struct ist prev_auth = authority;
-
-			if (http_authority_has_forbidden_char(list[hdr_idx].v) ||
-			    h3_set_authority(qcs, &authority, list[hdr_idx].v)) {
+			if (h3_set_authority(qcs, &authority, list[hdr_idx].v)) {
 				h3s->err = H3_ERR_MESSAGE_ERROR;
 				qcc_report_glitch(h3c->qcc, 1);
 				len = -1;
 				goto out;
 			}
-
-			if (isttest(prev_auth)) {
-				/* skip duplicate Host header */
-				++hdr_idx;
-				continue;
-			}
 		}
 		else if (isteq(list[hdr_idx].n, ist("cookie"))) {
 			http_cookie_register(list, hdr_idx, &cookie, &last_cookie);
@@ -1110,7 +1075,7 @@ static ssize_t h3_trailers_to_htx(struct qcs *qcs, const struct buffer *buf,
 
 		for (i = 0; i < list[hdr_idx].n.len; ++i) {
 			const char c = list[hdr_idx].n.ptr[i];
-			if ((uint8_t)(c - 'A') <= 'Z' - 'A' || !HTTP_IS_TOKEN(c)) {
+			if ((uint8_t)(c - 'A') < 'Z' - 'A' || !HTTP_IS_TOKEN(c)) {
 				TRACE_ERROR("invalid characters in field name", H3_EV_RX_FRAME|H3_EV_RX_HDR, qcs->qcc->conn, qcs);
 				h3s->err = H3_ERR_MESSAGE_ERROR;
 				qcc_report_glitch(h3c->qcc, 1);
@@ -1624,8 +1589,8 @@ static int h3_control_send(struct qcs *qcs, void *ctx)
 		goto err;
 	}
 
-	if (!(res = qcc_get_stream_txbuf(qcs, &err))) {
-		/* Consider alloc failure fatal for control stream even on conn buf limit. */
+	if (!(res = qcc_get_stream_txbuf(qcs, &err, 0))) {
+		/* Only memory failure can cause buf alloc error for control stream due to qcs_send_metadata() usage. */
 		TRACE_ERROR("cannot allocate Tx buffer", H3_EV_TX_FRAME|H3_EV_TX_SETTINGS, qcs->qcc->conn, qcs);
 		goto err;
 	}
@@ -1676,7 +1641,6 @@ static int h3_encode_header(struct buffer *buf,
 
 static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 {
-	struct h3s *h3s = qcs->ctx;
 	int err;
 	struct buffer outbuf;
 	struct buffer headers_buf = BUF_NULL;
@@ -1686,6 +1650,7 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 	struct htx_blk *blk;
 	enum htx_blk_type type;
 	int frame_length_size;  /* size in bytes of frame length varint field */
+	int smallbuf = 1;
 	int ret = 0;
 	int hdr;
 	int status = 0;
@@ -1707,20 +1672,8 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 			/* start-line -> HEADERS h3 frame */
 			BUG_ON(sl);
 			sl = htx_get_blk_ptr(htx, blk);
+			/* TODO should be on h3 layer */
 			status = sl->info.res.status;
-			if (status < 100 || status > 999) {
-				TRACE_ERROR("invalid response status code", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
-				goto err;
-			}
-			else if (status >= 100 && status < 200) {
-				TRACE_USER("handling interim HTX response", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
-				BUG_ON(conn_is_back(qcs->qcc->conn)); /* H3_SF_SENT_INTERIM is FE side only. */
-				h3s->flags |= H3_SF_SENT_INTERIM;
-			}
-			else {
-				TRACE_USER("handling final HTX response", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
-				h3s->flags &= ~H3_SF_SENT_INTERIM;
-			}
 		}
 		else if (type == HTX_BLK_HDR) {
 			if (unlikely(hdr >= sizeof(list) / sizeof(list[0]) - 1)) {
@@ -1738,45 +1691,39 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 		}
 	}
 
-	/* Current function expects HTX start-line to be present. This also
-	 * ensures <status> conformance has been checked prior to encoding it.
-	 */
 	BUG_ON(!sl);
 
 	list[hdr].n = ist("");
 
-	if (!(res = qcc_get_stream_txbuf(qcs, &err))) {
+ retry:
+	res = smallbuf ? qcc_get_stream_txbuf(qcs, &err, 1) :
+	                 qcc_realloc_stream_txbuf(qcs);
+	if (!res) {
 		if (err) {
 			TRACE_ERROR("cannot allocate Tx buffer", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
 			goto err;
 		}
 
-		TRACE_STATE("conn buf limit reached", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
+		TRACE_STATE("buf window full", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
 		goto end;
 	}
 
-	/* Reserve space for frame type + length as a max varint size. */
-	if (unlikely(b_contig_space(res) < 5)) {
-		/* Most of the times, h3_resp_headers_send() is only called one
-		 * time per stream, so buffer will be empty. However, this
-		 * assumption is invalid when handling interim responses, so
-		 * it's important to check out buffer remaining space.
-		 */
-		goto err;
-	}
+	/* Buffer allocated just now : must be enough for frame type + length as a max varint size */
+	BUG_ON(b_room(res) < 5);
 
 	b_reset(&outbuf);
 	outbuf = b_make(b_tail(res), b_contig_space(res), 0, 0);
 	/* Start the headers after frame type + length */
-	headers_buf = b_make(b_tail(res) + 5, b_contig_space(res) - 5, 0, 0);
+	headers_buf = b_make(b_head(res) + 5, b_size(res) - 5, 0, 0);
 
 	TRACE_DATA("encoding HEADERS frame", H3_EV_TX_FRAME|H3_EV_TX_HDR,
 	           qcs->qcc->conn, qcs);
 	if (qpack_encode_field_section_line(&headers_buf))
-		goto err;
+		goto err_full;
 	if (qpack_encode_int_status(&headers_buf, status)) {
+		/* TODO handle invalid status code VS no buf space left */
 		TRACE_ERROR("error during status code encoding", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
-		goto err;
+		goto err_full;
 	}
 
 	for (hdr = 0; hdr < sizeof(list) / sizeof(list[0]); ++hdr) {
@@ -1808,29 +1755,16 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 		}
 
 		if (h3_encode_header(&headers_buf, list[hdr].n, list[hdr].v))
-			goto err;
+			goto err_full;
 	}
 
 	/* Now that all headers are encoded, we are certain that res buffer is
 	 * big enough
 	 */
-
-	/* Encode H3 HEADERS frame type + frame length. If buffer is empty,
-	 * length is encoded on min varint size and buf head is realigned.
-	 * However, if it still contains a previously encoded interim response
-	 * not yet emitted, buf head must not be adjusted and frame length is
-	 * encoded as a 4-bytes varint so that responses are contiguous.
-	 */
-	if (!b_data(res)) {
-		frame_length_size = quic_int_getsize(b_data(&headers_buf));
-		res->head += 4 - frame_length_size;
-		b_putchr(res, 0x01); /* h3 HEADERS frame type */
-		b_quic_enc_int(res, b_data(&headers_buf), 0);
-	}
-	else {
-		b_putchr(res, 0x01); /* h3 HEADERS frame type */
-		b_quic_enc_int(res, b_data(&headers_buf), 4);
-	}
+	frame_length_size = quic_int_getsize(b_data(&headers_buf));
+	res->head += 4 - frame_length_size;
+	b_putchr(res, 0x01); /* h3 HEADERS frame type */
+	b_quic_enc_int(res, b_data(&headers_buf), 0);
 	b_add(res, b_data(&headers_buf));
 
 	ret = 0;
@@ -1847,6 +1781,12 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 	TRACE_LEAVE(H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
 	return ret;
 
+ err_full:
+	if (smallbuf) {
+		TRACE_DEVEL("retry with a full buffer", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
+		smallbuf = 0;
+		goto retry;
+	}
  err:
 	TRACE_DEVEL("leaving on error", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
 	return -1;
@@ -1923,13 +1863,13 @@ static int h3_resp_trailers_send(struct qcs *qcs, struct htx *htx)
 	list[hdr].n = ist("");
 
  start:
-	if (!(res = qcc_get_stream_txbuf(qcs, &err))) {
+	if (!(res = qcc_get_stream_txbuf(qcs, &err, 0))) {
 		if (err) {
 			TRACE_ERROR("cannot allocate Tx buffer", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
 			goto err;
 		}
 
-		TRACE_STATE("conn buf limit reached", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
+		TRACE_STATE("buf window full", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
 		goto end;
 	}
 
@@ -2058,14 +1998,13 @@ static int h3_resp_data_send(struct qcs *qcs, struct htx *htx,
 	if (type != HTX_BLK_DATA)
 		goto end;
 
-	if (!(res = qcc_get_stream_txbuf(qcs, &err))) {
+	if (!(res = qcc_get_stream_txbuf(qcs, &err, 0))) {
 		if (err) {
 			TRACE_ERROR("cannot allocate Tx buffer", H3_EV_TX_FRAME|H3_EV_TX_DATA, qcs->qcc->conn, qcs);
 			goto err;
 		}
 
-		/* Connection buf limit reached, stconn will subscribe on SEND. */
-		TRACE_STATE("conn buf limit reached", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
+		TRACE_STATE("buf window full", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
 		goto end;
 	}
 
@@ -2155,24 +2094,19 @@ static int h3_resp_data_send(struct qcs *qcs, struct htx *htx,
 	return -1;
 }
 
-static size_t h3_snd_buf(struct qcs *qcs, struct buffer *buf, size_t count, char *fin)
+static size_t h3_snd_buf(struct qcs *qcs, struct buffer *buf, size_t count)
 {
-	struct h3s *h3s = qcs->ctx;
 	size_t total = 0;
 	enum htx_blk_type btype;
 	struct htx *htx;
 	struct htx_blk *blk;
 	uint32_t bsize;
 	int32_t idx;
-	char eom;
 	int ret = 0;
 
 	TRACE_ENTER(H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
 
-	*fin = 0;
 	htx = htx_from_buf(buf);
-	/* EOM is saved here, useful if 0-copy is performed with HTX buf. */
-	eom = htx->flags & HTX_FL_EOM;
 
 	while (count && !htx_is_empty(htx) && qcc_stream_can_send(qcs) && ret >= 0) {
 		idx = htx_get_head(htx);
@@ -2185,7 +2119,7 @@ static size_t h3_snd_buf(struct qcs *qcs, struct buffer *buf, size_t count, char
 
 		switch (btype) {
 		case HTX_BLK_RES_SL:
-			/* start-line -> HEADERS h3 frame (FE side) */
+			/* start-line -> HEADERS h3 frame */
 			ret = h3_resp_headers_send(qcs, htx);
 			if (ret > 0) {
 				total += ret;
@@ -2266,10 +2200,6 @@ static size_t h3_snd_buf(struct qcs *qcs, struct buffer *buf, size_t count, char
 #endif
 
  out:
-	if (eom && htx_is_empty(htx) && !(h3s->flags & H3_SF_SENT_INTERIM)) {
-		TRACE_USER("transcoding last HTX message", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
-		*fin = 1;
-	}
 	htx_to_buf(htx, buf);
 
 	TRACE_LEAVE(H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
@@ -2286,7 +2216,7 @@ static size_t h3_nego_ff(struct qcs *qcs, size_t count)
 	TRACE_ENTER(H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
 
  start:
-	if (!(res = qcc_get_stream_txbuf(qcs, &err))) {
+	if (!(res = qcc_get_stream_txbuf(qcs, &err, 0))) {
 		if (err) {
 			qcs->sd->iobuf.flags |= IOBUF_FL_NO_FF;
 			goto end;
@@ -2496,7 +2426,7 @@ static int h3_send_goaway(struct h3c *h3c)
 	b_quic_enc_int(&pos, frm_len, 0);
 	b_quic_enc_int(&pos, h3c->id_goaway, 0);
 
-	res = qcc_get_stream_txbuf(qcs, &err);
+	res = qcc_get_stream_txbuf(qcs, &err, 0);
 	if (!res || b_room(res) < b_data(&pos) ||
 	    qfctl_sblocked(&qcs->tx.fc) || qfctl_sblocked(&h3c->qcc->tx.fc)) {
 		/* Do not try forcefully to emit GOAWAY if no buffer available or not enough space left. */
@@ -2569,21 +2499,6 @@ static int h3_finalize(void *ctx)
 
 	TRACE_ENTER(H3_EV_H3C_NEW, qcc->conn);
 
-	/* RFC 9114 6.2. Unidirectional Streams
-	 *
-	 * Each endpoint needs to create at least one unidirectional stream for
-	 * the HTTP control stream. QPACK requires two additional unidirectional
-	 * streams, and other extensions might require further streams.
-	 * Therefore, the transport parameters sent by both clients and servers
-	 * MUST allow the peer to create at least three unidirectional streams.
-	 */
-	if (qcc_fctl_avail_streams(qcc, 0) < 3) {
-		TRACE_ERROR("peer flow-control limit does not allow control stream creation", H3_EV_H3C_NEW, qcc->conn);
-		qcc_set_error(qcc, H3_ERR_GENERAL_PROTOCOL_ERROR, 1);
-		qcc_report_glitch(qcc, 1);
-		goto err;
-	}
-
 	qcs = qcc_init_stream_local(qcc, 0);
 	if (!qcs) {
 		/* Error must be set by qcc_init_stream_local(). */
@@ -2592,6 +2507,7 @@ static int h3_finalize(void *ctx)
 		goto err;
 	}
 
+	qcs_send_metadata(qcs);
 	h3c->ctrl_strm = qcs;
 
 	if (h3_control_send(qcs, h3c) < 0) {
