@@ -825,7 +825,7 @@ int create_listeners(struct bind_conf *bc, const struct sockaddr_storage *ss,
 		proto->add(proto, l);
 
 		if (fd != -1)
-			l->rx.flags |= RX_F_INHERITED_FD|RX_F_INHERITED_SOCK;
+			l->rx.flags |= RX_F_INHERITED;
 
 		guid_init(&l->guid);
 
@@ -1034,7 +1034,7 @@ void listener_accept(struct listener *l)
 	int ret;
 
 	p = l->bind_conf->frontend;
-	bind_tid_commit = l->rx.proto ? l->rx.proto->bind_tid_commit : NULL;
+	bind_tid_commit = l->rx.proto->bind_tid_commit;
 
 	/* if l->bind_conf->maxaccept is -1, then max_accept is UINT_MAX. It is
 	 * not really illimited, but it is probably enough.
@@ -1473,7 +1473,7 @@ void listener_accept(struct listener *l)
 			 * reservation in the target ring.
 			 */
 
-			if (l->rx.proto && l->rx.proto->bind_tid_prep) {
+			if (l->rx.proto->bind_tid_prep) {
 				if (l->rx.proto->bind_tid_prep(cli_conn, t)) {
 					/* Failed migration, stay on the same thread. */
 					goto local_accept;
@@ -1488,11 +1488,6 @@ void listener_accept(struct listener *l)
 			 */
 			ring = &accept_queue_rings[t];
 			if (accept_queue_push_mp(ring, cli_conn, bind_tid_commit)) {
-				if (new_li) {
-					_HA_ATOMIC_INC(&new_li->nbconn);
-					_HA_ATOMIC_DEC(&l->nbconn);
-				}
-
 				_HA_ATOMIC_INC(&activity[t].accq_pushed);
 				tasklet_wakeup(ring->tasklet);
 
@@ -1503,7 +1498,7 @@ void listener_accept(struct listener *l)
 			 */
 			_HA_ATOMIC_INC(&activity[t].accq_full);
 
-			if (l->rx.proto && l->rx.proto->bind_tid_reset)
+			if (l->rx.proto->bind_tid_reset)
 				l->rx.proto->bind_tid_reset(cli_conn);
 		}
 #endif // USE_THREAD
@@ -1578,7 +1573,7 @@ void listener_accept(struct listener *l)
 	/* This may be a shared socket that was paused by another process.
 	 * Let's put it to pause in this case.
 	 */
-	if (l->rx.proto && l->rx.proto->rx_listening(&l->rx) == 0) {
+	if (l->rx.proto->rx_listening(&l->rx) == 0) {
 		suspend_listener(l, 0, 0);
 		goto end;
 	}
@@ -1804,12 +1799,6 @@ int bind_complete_thread_setup(struct bind_conf *bind_conf, int *err_code)
 					/* it has been allocated already in the previous round */
 					shard_info_attach(&new_li->rx, ref->rx.shard_info);
 					new_li->rx.flags |= RX_F_MUST_DUP;
-					/* taking the other one's FD will result in it being marked
-					 * extern and being dup()ed. Let's mark the receiver as
-					 * inherited so that it properly bypasses all second-stage
-					 * setup/unbind and avoids being passed to new processes.
-					 */
-					new_li->rx.flags |= ref->rx.flags & RX_F_INHERITED_SOCK;
 				}
 
 				gmask &= gmask - 1; // drop lowest bit
@@ -1842,7 +1831,7 @@ int bind_complete_thread_setup(struct bind_conf *bind_conf, int *err_code)
 					new_li->luid = new_li->conf.id.key = tmp_li->luid;
 					tmp_li->luid = 0;
 					eb32_delete(&tmp_li->conf.id);
-					if (new_li->luid)
+					if (tmp_li->luid)
 						eb32_insert(&fe->conf.used_listener_id, &new_li->conf.id);
 					new_li = tmp_li;
 				}
@@ -1866,7 +1855,7 @@ int bind_complete_thread_setup(struct bind_conf *bind_conf, int *err_code)
 			new_li->luid = new_li->conf.id.key = li->luid;
 			li->luid = 0;
 			eb32_delete(&li->conf.id);
-			if (new_li->luid)
+			if (li->luid)
 				eb32_insert(&fe->conf.used_listener_id, &new_li->conf.id);
 		}
 	}
@@ -2050,8 +2039,7 @@ struct bind_conf *bind_conf_alloc(struct proxy *fe, const char *file,
 #ifdef USE_QUIC
 	/* Use connection socket for QUIC by default. */
 	bind_conf->quic_mode = QUIC_SOCK_MODE_CONN;
-	bind_conf->max_cwnd =
-		global.tune.bufsize * global.tune.quic_streams_buf;
+	bind_conf->max_cwnd = global.tune.quic_frontend_max_window_size;
 #endif
 	LIST_INIT(&bind_conf->listeners);
 
