@@ -21,6 +21,7 @@
  */
 
 #include <haproxy/quic_cc.h>
+#include <haproxy/quic_pacing.h>
 
 struct quic_cc_algo *default_quic_cc_algo = &quic_cc_algo_cubic;
 
@@ -40,7 +41,8 @@ void quic_cc_init(struct quic_cc *cc,
 /* Send <ev> event to <cc> congestion controller. */
 void quic_cc_event(struct quic_cc *cc, struct quic_cc_event *ev)
 {
-	cc->algo->event(cc, ev);
+	if (cc->algo->event)
+		cc->algo->event(cc, ev);
 }
 
 void quic_cc_state_trace(struct buffer *buf, const struct quic_cc *cc)
@@ -48,8 +50,15 @@ void quic_cc_state_trace(struct buffer *buf, const struct quic_cc *cc)
 	cc->algo->state_trace(buf, cc);
 }
 
+/* Return interval in nanoseconds between each datagram emission for a smooth pacing. */
+uint quic_cc_default_pacing_inter(const struct quic_cc *cc)
+{
+	struct quic_cc_path *path = container_of(cc, struct quic_cc_path, cc);
+	return path->loss.srtt * 1000000 / (path->cwnd / path->mtu + 1) + 1;
+}
+
 /* Returns true if congestion window on path ought to be increased. */
-static int quic_cwnd_may_increase(const struct quic_cc_path *path)
+int quic_cwnd_may_increase(const struct quic_cc_path *path)
 {
 	/* RFC 9002 7.8. Underutilizing the Congestion Window
 	 *
@@ -66,32 +75,3 @@ static int quic_cwnd_may_increase(const struct quic_cc_path *path)
 	 */
 	return 2 * path->in_flight >= path->cwnd  || path->cwnd < 16384;
 }
-
-/* Restore congestion window for <path> to its minimal value. */
-void quic_cc_path_reset(struct quic_cc_path *path)
-{
-	path->cwnd = path->limit_min;
-}
-
-/* Set congestion window for <path> to <val>. Min and max limits are enforced. */
-void quic_cc_path_set(struct quic_cc_path *path, uint64_t val)
-{
-	path->cwnd = QUIC_MIN(val, path->limit_max);
-	path->cwnd = QUIC_MAX(path->cwnd, path->limit_min);
-
-	path->cwnd_last_max = QUIC_MAX(path->cwnd, path->cwnd_last_max);
-}
-
-/* Increment congestion window for <path> with <val>. Min and max limits are
- * enforced. Contrary to quic_cc_path_set(), increase is performed only if a
- * certain minimal level of the window was already filled.
- */
-void quic_cc_path_inc(struct quic_cc_path *path, uint64_t val)
-{
-	if (quic_cwnd_may_increase(path)) {
-		path->cwnd = QUIC_MIN(path->cwnd + val, path->limit_max);
-		path->cwnd = QUIC_MAX(path->cwnd, path->limit_min);
-
-		path->cwnd_last_max = QUIC_MAX(path->cwnd, path->cwnd_last_max);
-	}
-}
