@@ -75,6 +75,21 @@ static int cli_parse_show_quic(char **args, char *payload, struct appctx *appctx
 		ctx->fields = QUIC_DUMP_FLD_MASK;
 		++argc;
 	}
+	else if (strcmp(args[argc], "help") == 0) {
+		chunk_printf(&trash,
+			     "Usage: show quic [help|<format>] [<filter>]\n"
+			     "Dumps information about QUIC connections. Available output formats:\n"
+			     "  oneline    dump a single, netstat-like line per connection (default)\n"
+			     "  full       dump all known information about each connection\n"
+			     "  <levels>*  only dump certain information, defined by a comma-delimited list\n"
+			     "             of levels among 'tp', 'sock', 'pktns', 'cc', or 'mux'\n"
+			     "  help       display this help\n"
+			     "Available output filters:\n"
+			     "  all        dump all connections (the default)\n"
+			     "  <id>       dump only the connection matching this identifier (0x...)\n"
+			     "Without any argument, all connections are dumped using the oneline format.\n");
+		return cli_err(appctx, trash.area);
+	}
 	else if (*args[argc]) {
 		struct ist istarg = ist(args[argc]);
 		struct ist field = istsplit(&istarg, ',');
@@ -101,7 +116,7 @@ static int cli_parse_show_quic(char **args, char *payload, struct appctx *appctx
 				 * field name has been specified.
 				 */
 				if (istarg.len || ctx->fields) {
-					cli_err(appctx, "Invalid field.\n");
+					cli_err(appctx, "Invalid field, use 'help' for more options.\n");
 					return 1;
 				}
 
@@ -140,7 +155,7 @@ static int cli_parse_show_quic(char **args, char *payload, struct appctx *appctx
 			ctx->flags |= QC_CLI_FL_SHOW_ALL;
 		}
 		else {
-			cli_err(appctx, "Invalid argument.\n");
+			cli_err(appctx, "Invalid argument, use 'help' for more options.\n");
 			return 1;
 		}
 
@@ -306,11 +321,14 @@ static void dump_quic_full(struct show_quic_ctx *ctx, struct quic_conn *qc)
 		if (qc->path->cc.algo->state_cli)
 			qc->path->cc.algo->state_cli(&trash, qc->path);
 
-		chunk_appendf(&trash, "  srtt=%-4u rttvar=%-4u rttmin=%-4u ptoc=%-4u cwnd=%-6llu"
-		                      " cwnd_last_max=%-6llu sentpkts=%-6llu lostpkts=%-6llu reorderedpkts=%-6llu\n",
+		chunk_appendf(&trash, "  srtt=%-4u  rttvar=%-4u rttmin=%-4u ptoc=%-4u\n"
+		                      "  cwnd=%-6llu            mcwnd=%-6llu\n"
+		                      "  sentbytes=%-12llu sentbytesgso=%-12llu sentpkts=%-6llu\n"
+		                      "  lostpkts=%-6llu        reorderedpkts=%-6llu\n",
 		              qc->path->loss.srtt, qc->path->loss.rtt_var,
 		              qc->path->loss.rtt_min, qc->path->loss.pto_count, (ullong)qc->path->cwnd,
-		              (ullong)qc->path->cwnd_last_max, (ullong)qc->cntrs.sent_pkt, (ullong)qc->path->loss.nb_lost_pkt, (ullong)qc->path->loss.nb_reordered_pkt);
+		              (ullong)qc->path->mcwnd, (ullong)qc->cntrs.sent_bytes, (ullong)qc->cntrs.sent_bytes_gso,
+		              (ullong)qc->cntrs.sent_pkt, (ullong)qc->path->loss.nb_lost_pkt, (ullong)qc->path->loss.nb_reordered_pkt);
 	}
 
 	if (qc->cntrs.dropped_pkt) {
@@ -392,10 +410,7 @@ static int cli_io_handler_dump_quic(struct appctx *appctx)
 				      "in_flight infl_p lost_p         "
 				      "Local Address           Foreign Address      "
 				      "local & remote CIDs\n");
-			if (applet_putchk(appctx, &trash) == -1) {
-				/* Trash buf too short without any data emitted yet, should never happen. */
-				goto done;
-			}
+			applet_putchk(appctx, &trash);
 		}
 	}
 
@@ -489,7 +504,7 @@ static void cli_release_show_quic(struct appctx *appctx)
 }
 
 static struct cli_kw_list cli_kws = {{ }, {
-	{ { "show", "quic", NULL }, "show quic [<format>] [<filter>]         : display quic connections status", cli_parse_show_quic, cli_io_handler_dump_quic, cli_release_show_quic },
+	{ { "show", "quic", NULL }, "show quic [help|<format>] [<filter>]    : display quic connections status", cli_parse_show_quic, cli_io_handler_dump_quic, cli_release_show_quic },
 	{{},}
 }};
 
