@@ -115,7 +115,7 @@ void chunk_frm_appendf(struct buffer *buf, const struct quic_frame *frm)
 	{
 		const struct qf_crypto *crypto_frm = &frm->crypto;
 		chunk_appendf(buf, " cfoff=%llu cflen=%llu",
-		              (ull)crypto_frm->offset_node.key, (ull)crypto_frm->len);
+		              (ull)crypto_frm->offset, (ull)crypto_frm->len);
 		break;
 	}
 	case QUIC_FT_RESET_STREAM:
@@ -138,7 +138,7 @@ void chunk_frm_appendf(struct buffer *buf, const struct quic_frame *frm)
 		chunk_appendf(&trace_buf, " uni=%d fin=%d id=%llu off=%llu len=%llu",
 		              !!(strm_frm->id & QUIC_STREAM_FRAME_ID_DIR_BIT),
 		              !!(frm->type & QUIC_STREAM_FRAME_TYPE_FIN_BIT),
-		              (ull)strm_frm->id, (ull)strm_frm->offset.key, (ull)strm_frm->len);
+		              (ull)strm_frm->id, (ull)strm_frm->offset, (ull)strm_frm->len);
 		break;
 	}
 	case QUIC_FT_MAX_DATA:
@@ -363,16 +363,11 @@ static int quic_parse_ack_ecn_frame(struct quic_frame *frm, struct quic_conn *qc
                                     const unsigned char **pos, const unsigned char *end)
 {
 	struct qf_ack *ack_frm = &frm->ack;
-	/* TODO implement ECN advertising */
-	uint64_t ect0, ect1, ecn_ce;
 
 	return quic_dec_int(&ack_frm->largest_ack, pos, end) &&
-	       quic_dec_int(&ack_frm->ack_delay, pos, end) &&
-	       quic_dec_int(&ack_frm->ack_range_num, pos, end) &&
-	       quic_dec_int(&ack_frm->first_ack_range, pos, end) &&
-	       quic_dec_int(&ect0, pos, end) &&
-	       quic_dec_int(&ect1, pos, end) &&
-	       quic_dec_int(&ecn_ce, pos, end);
+		quic_dec_int(&ack_frm->ack_delay, pos, end) &&
+		quic_dec_int(&ack_frm->first_ack_range, pos, end) &&
+		quic_dec_int(&ack_frm->ack_range_num, pos, end);
 }
 
 /* Encode a RESET_STREAM frame at <pos> buffer position.
@@ -435,12 +430,12 @@ static int quic_build_crypto_frame(unsigned char **pos, const unsigned char *end
 	const struct quic_enc_level *qel = crypto_frm->qel;
 	size_t offset, len;
 
-	if (!quic_enc_int(pos, end, crypto_frm->offset_node.key) ||
+	if (!quic_enc_int(pos, end, crypto_frm->offset) ||
 	    !quic_enc_int(pos, end, crypto_frm->len) || end - *pos < crypto_frm->len)
 		return 0;
 
 	len = crypto_frm->len;
-	offset = crypto_frm->offset_node.key;
+	offset = crypto_frm->offset;
 	while (len) {
 		int idx;
 		size_t to_copy;
@@ -468,7 +463,7 @@ static int quic_parse_crypto_frame(struct quic_frame *frm, struct quic_conn *qc,
 {
 	struct qf_crypto *crypto_frm = &frm->crypto;
 
-	if (!quic_dec_int((uint64_t *)&crypto_frm->offset_node.key, pos, end) ||
+	if (!quic_dec_int(&crypto_frm->offset, pos, end) ||
 	    !quic_dec_int(&crypto_frm->len, pos, end) || end - *pos < crypto_frm->len)
 		return 0;
 
@@ -526,10 +521,10 @@ static int quic_build_stream_frame(unsigned char **pos, const unsigned char *end
 
 	/* Caller must set OFF bit if and only if a non-null offset is used. */
 	BUG_ON(!!(frm->type & QUIC_STREAM_FRAME_TYPE_OFF_BIT) !=
-	       !!strm_frm->offset.key);
+	       !!strm_frm->offset);
 
 	if (!quic_enc_int(pos, end, strm_frm->id) ||
-	    ((frm->type & QUIC_STREAM_FRAME_TYPE_OFF_BIT) && !quic_enc_int(pos, end, strm_frm->offset.key)) ||
+	    ((frm->type & QUIC_STREAM_FRAME_TYPE_OFF_BIT) && !quic_enc_int(pos, end, strm_frm->offset)) ||
 	    ((frm->type & QUIC_STREAM_FRAME_TYPE_LEN_BIT) &&
 	     (!quic_enc_int(pos, end, strm_frm->len) || end - *pos < strm_frm->len)))
 		return 0;
@@ -568,10 +563,9 @@ static int quic_parse_stream_frame(struct quic_frame *frm, struct quic_conn *qc,
 		return 0;
 
 	/* Offset parsing */
-	if (!(frm->type & QUIC_STREAM_FRAME_TYPE_OFF_BIT)) {
-		strm_frm->offset.key = 0;
-	}
-	else if (!quic_dec_int((uint64_t *)&strm_frm->offset.key, pos, end))
+	if (!(frm->type & QUIC_STREAM_FRAME_TYPE_OFF_BIT))
+		strm_frm->offset = 0;
+	else if (!quic_dec_int((uint64_t *)&strm_frm->offset, pos, end))
 		return 0;
 
 	/* Length parsing */
@@ -1247,6 +1241,12 @@ void qc_release_frm(struct quic_conn *qc, struct quic_frame *frm)
 	uint64_t pn;
 	struct quic_frame *origin, *f, *tmp;
 
+	/* <frm> will be detached from its Tx packet via origin->reflist loop
+	 * implemented below. It is thus expected that its pkt field is not
+	 * NULL or else it may free the frame too soon.
+	 */
+	BUG_ON(!frm->pkt);
+
 	TRACE_ENTER(QUIC_EV_CONN_PRSAFRM, qc, frm);
 
 	/* Identify this frame: a frame copy or one of its copies */
@@ -1285,3 +1285,4 @@ void qc_release_frm(struct quic_conn *qc, struct quic_frame *frm)
 
 	TRACE_LEAVE(QUIC_EV_CONN_PRSAFRM, qc);
 }
+
