@@ -1,5 +1,5 @@
 #include <haproxy/errors.h>
-#include <haproxy/ncbmbuf.h>
+#include <haproxy/ncbuf.h>
 #include <haproxy/proxy.h>
 #include <haproxy/quic_conn.h>
 #include <haproxy/quic_sock.h>
@@ -8,17 +8,11 @@
 #include <haproxy/quic_tp.h>
 #include <haproxy/quic_trace.h>
 #include <haproxy/ssl_sock.h>
-#include <haproxy/stats.h>
 #include <haproxy/trace.h>
 
+static BIO_METHOD *ha_quic_meth;
+
 DECLARE_POOL(pool_head_quic_ssl_sock_ctx, "quic_ssl_sock_ctx", sizeof(struct ssl_sock_ctx));
-const char *default_quic_ciphersuites = "TLS_AES_128_GCM_SHA256:TLS_AES_256_GCM_SHA384"
-                           ":TLS_CHACHA20_POLY1305_SHA256:TLS_AES_128_CCM_SHA256";
-#ifdef HAVE_OPENSSL_QUIC
-const char *default_quic_curves = "X25519:P-256:P-384:P-521:X25519MLKEM768";
-#else
-const char *default_quic_curves = "X25519:P-256:P-384:P-521";
-#endif
 
 /* Set the encoded version of the transport parameter into the TLS
  * stack depending on <ver> QUIC version and <server> boolean which must
@@ -149,7 +143,7 @@ static int qc_ssl_crypto_data_cpy(struct quic_conn *qc, struct quic_enc_level *q
 				goto leave;
 			}
 
-			frm->crypto.offset_node.key = cf_offset;
+			frm->crypto.offset = cf_offset;
 			frm->crypto.len = cf_len;
 			frm->crypto.qel = qel;
 			LIST_APPEND(&qel->pktns->tx.frms, &frm->list);
@@ -223,7 +217,7 @@ static int ha_quic_set_encryption_secrets(SSL *ssl, enum ssl_encryption_level_t
 		goto leave;
 	}
 
-	if (!quic_tls_dec_aes_ctx_init(&rx->hp_ctx, rx->hp, rx->hp_key)) {
+	if (!quic_tls_dec_hp_ctx_init(&rx->hp_ctx, rx->hp, rx->hp_key)) {
 		TRACE_ERROR("could not initial RX TLS cipher context for HP", QUIC_EV_CONN_RWSEC, qc);
 		goto leave;
 	}
@@ -275,7 +269,7 @@ write:
 		goto leave;
 	}
 
-	if (!quic_tls_enc_aes_ctx_init(&tx->hp_ctx, tx->hp, tx->hp_key)) {
+	if (!quic_tls_enc_hp_ctx_init(&tx->hp_ctx, tx->hp, tx->hp_key)) {
 		TRACE_ERROR("could not initial TX TLS cipher context for HP", QUIC_EV_CONN_RWSEC, qc);
 		goto leave;
 	}
@@ -446,11 +440,8 @@ static SSL_QUIC_METHOD ha_quic_method = {
  */
 int ssl_quic_initial_ctx(struct bind_conf *bind_conf)
 {
+	struct ssl_bind_conf __maybe_unused *ssl_conf_cur;
 	int cfgerr = 0;
-	const char *ciphersuites = bind_conf->ssl_conf.ciphersuites ?
-		bind_conf->ssl_conf.ciphersuites : default_quic_ciphersuites;
-	const char *curves = bind_conf->ssl_conf.curves ?
-		bind_conf->ssl_conf.curves : default_quic_curves;
 
 	long options =
 		(SSL_OP_ALL & ~SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS) |
@@ -468,29 +459,11 @@ int ssl_quic_initial_ctx(struct bind_conf *bind_conf)
 	SSL_CTX_set_min_proto_version(ctx, TLS1_3_VERSION);
 	SSL_CTX_set_max_proto_version(ctx, TLS1_3_VERSION);
 
-    if (SSL_CTX_set_ciphersuites(ctx, ciphersuites) != 1) {
-        ha_warning("Binding [%s:%d] for %s %s: default QUIC cipher"
-                   " suites setting failed.\n",
-                   bind_conf->file, bind_conf->line,
-                   proxy_type_str(bind_conf->frontend),
-                   bind_conf->frontend->id);
-        cfgerr++;
-    }
-
-    if (SSL_CTX_set1_curves_list(ctx, curves) != 1) {
-        ha_warning("Binding [%s:%d] for %s %s: default QUIC cipher"
-                   " curves setting failed.\n",
-                   bind_conf->file, bind_conf->line,
-                   proxy_type_str(bind_conf->frontend),
-                   bind_conf->frontend->id);
-        cfgerr++;
-    }
-
 	if (bind_conf->ssl_conf.early_data) {
 #if !defined(HAVE_SSL_0RTT_QUIC)
 		ha_warning("Binding [%s:%d] for %s %s: 0-RTT with QUIC is not supported by this SSL library, ignored.\n",
 		           bind_conf->file, bind_conf->line, proxy_type_str(bind_conf->frontend), bind_conf->frontend->id);
-#elif defined(OPENSSL_IS_BORINGSSL) || defined(USE_OPENSSL_AWSLC)
+#elif defined(OPENSSL_IS_BORINGSSL) || defined(OPENSSL_IS_AWSLC)
 		SSL_CTX_set_early_data_enabled(ctx, 1);
 #else
 		SSL_CTX_set_options(ctx, SSL_OP_NO_ANTI_REPLAY);
@@ -499,7 +472,10 @@ int ssl_quic_initial_ctx(struct bind_conf *bind_conf)
 	}
 
 #ifdef SSL_CTRL_SET_TLSEXT_HOSTNAME
-# if defined(HAVE_SSL_CLIENT_HELLO_CB)
+# if defined(OPENSSL_IS_BORINGSSL) || defined(OPENSSL_IS_AWSLC)
+	SSL_CTX_set_select_certificate_cb(ctx, ssl_sock_switchctx_cbk);
+	SSL_CTX_set_tlsext_servername_callback(ctx, ssl_sock_switchctx_err_cbk);
+# elif defined(HAVE_SSL_CLIENT_HELLO_CB)
 	SSL_CTX_set_client_hello_cb(ctx, ssl_sock_switchctx_cbk, NULL);
 	SSL_CTX_set_tlsext_servername_callback(ctx, ssl_sock_switchctx_err_cbk);
 # else /* ! HAVE_SSL_CLIENT_HELLO_CB */
@@ -542,7 +518,7 @@ static forceinline void qc_ssl_dump_errors(struct connection *conn)
  * Remaining parameter are there for debugging purposes.
  * Return 1 if succeeded, 0 if not.
  */
-static int qc_ssl_provide_quic_data(struct ncbmbuf *ncbuf,
+static int qc_ssl_provide_quic_data(struct ncbuf *ncbuf,
                                     enum ssl_encryption_level_t level,
                                     struct ssl_sock_ctx *ctx,
                                     const unsigned char *data, size_t len)
@@ -553,18 +529,12 @@ static int qc_ssl_provide_quic_data(struct ncbmbuf *ncbuf,
 	int ssl_err, state;
 	struct quic_conn *qc;
 	int ret = 0;
-	struct ssl_counters *counters = NULL;
-	struct ssl_counters *counters_px = NULL;
 
 	ssl_err = SSL_ERROR_NONE;
 	qc = ctx->qc;
 
 	TRACE_ENTER(QUIC_EV_CONN_SSLDATA, qc);
 
-	counters = EXTRA_COUNTERS_GET(qc->li->extra_counters, &ssl_stats_module);
-	counters_px = EXTRA_COUNTERS_GET(qc->li->bind_conf->frontend->extra_counters_fe,
-	                                 &ssl_stats_module);
-
 	if (SSL_provide_quic_data(ctx->ssl, level, data, len) != 1) {
 		TRACE_ERROR("SSL_provide_quic_data() error",
 		            QUIC_EV_CONN_SSLDATA, qc, NULL, NULL, ctx->ssl);
@@ -604,18 +574,17 @@ static int qc_ssl_provide_quic_data(struct ncbmbuf *ncbuf,
 			ERR_clear_error();
 			goto leave;
 		}
+#if defined(LIBRESSL_VERSION_NUMBER)
 		else if (qc->flags & QUIC_FL_CONN_IMMEDIATE_CLOSE) {
-			/* Immediate close may be set due to invalid received
-			 * transport parameters. This is also used due to some
-			 * SSL libraries which emit TLS alerts without failing
-			 * on SSL_do_handshake(). This is at least the case for
-			 * libressl-3.9.0 when forcing the TLS cipher to
-			 * TLS_AES_128_CCM_SHA256.
+			/* Some libressl versions emit TLS alerts without making the handshake
+			 * (SSL_do_handshake()) fail. This is at least the case for
+			 * libressl-3.9.0 when forcing the TLS cipher to TLS_AES_128_CCM_SHA256.
 			 */
 			TRACE_ERROR("SSL handshake error", QUIC_EV_CONN_IO_CB, qc, &state, &ssl_err);
 			HA_ATOMIC_INC(&qc->prx_counters->hdshk_fail);
 			goto leave;
 		}
+#endif
 
 #if defined(OPENSSL_IS_AWSLC)
 		/* As a server, if early data is accepted, SSL_do_handshake will
@@ -640,7 +609,6 @@ static int qc_ssl_provide_quic_data(struct ncbmbuf *ncbuf,
 #endif
 
 		TRACE_PROTO("SSL handshake OK", QUIC_EV_CONN_IO_CB, qc, &state);
-		ssl_sock_update_counters(ctx->ssl, counters, counters_px, 0);
 
 		/* Check the alpn could be negotiated */
 		if (!qc->app_ops) {
@@ -702,12 +670,17 @@ static int qc_ssl_provide_quic_data(struct ncbmbuf *ncbuf,
 	/* The CRYPTO data are consumed even in case of an error to release
 	 * the memory asap.
 	 */
-	if (!ncbmb_is_null(ncbuf)) {
+	if (!ncb_is_null(ncbuf)) {
 #ifdef DEBUG_STRICT
-		ncb_ret = ncbmb_advance(ncbuf, len);
+		ncb_ret = ncb_advance(ncbuf, len);
+		/* ncb_advance() must always succeed. This is guaranteed as
+		 * this is only done inside a data block. If false, this will
+		 * lead to handshake failure with quic_enc_level offset shifted
+		 * from buffer data.
+		 */
 		BUG_ON(ncb_ret != NCB_RET_OK);
 #else
-		ncbmb_advance(ncbuf, len);
+		ncb_advance(ncbuf, len);
 #endif
 	}
 
@@ -722,7 +695,7 @@ int qc_ssl_provide_all_quic_data(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
 {
 	int ret = 0;
 	struct quic_enc_level *qel;
-	struct ncbmbuf *ncbuf;
+	struct ncbuf *ncbuf;
 	ncb_sz_t data;
 
 	TRACE_ENTER(QUIC_EV_CONN_PHPKTS, qc);
@@ -733,12 +706,12 @@ int qc_ssl_provide_all_quic_data(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
 			continue;
 
 		ncbuf = &cstream->rx.ncbuf;
-		if (ncbmb_is_null(ncbuf))
+		if (ncb_is_null(ncbuf))
 			continue;
 
 		/* TODO not working if buffer is wrapping */
-		while ((data = ncbmb_data(ncbuf, 0))) {
-			const unsigned char *cdata = (const unsigned char *)ncbmb_head(ncbuf);
+		while ((data = ncb_data(ncbuf, 0))) {
+			const unsigned char *cdata = (const unsigned char *)ncb_head(ncbuf);
 
 			if (!qc_ssl_provide_quic_data(&qel->cstream->rx.ncbuf, qel->level,
 			                              ctx, cdata, data))
@@ -749,7 +722,7 @@ int qc_ssl_provide_all_quic_data(struct quic_conn *qc, struct ssl_sock_ctx *ctx)
 			            QUIC_EV_CONN_PHPKTS, qc, qel);
 		}
 
-		if (!ncbmb_is_null(ncbuf) && ncbmb_is_empty(ncbuf)) {
+		if (!ncb_is_null(ncbuf) && ncb_is_empty(ncbuf)) {
 			TRACE_DEVEL("freeing crypto buf", QUIC_EV_CONN_PHPKTS, qc, qel);
 			quic_free_ncbuf(ncbuf);
 		}
@@ -780,7 +753,7 @@ static int qc_ssl_sess_init(struct quic_conn *qc, SSL_CTX *ssl_ctx, SSL **ssl)
 	*ssl = SSL_new(ssl_ctx);
 	if (!*ssl) {
 		if (!retry--)
-			goto err;
+			goto leave;
 
 		pool_gc(NULL);
 		goto retry;
@@ -791,7 +764,7 @@ static int qc_ssl_sess_init(struct quic_conn *qc, SSL_CTX *ssl_ctx, SSL **ssl)
 		SSL_free(*ssl);
 		*ssl = NULL;
 		if (!retry--)
-			goto err;
+			goto leave;
 
 		pool_gc(NULL);
 		goto retry;
@@ -801,9 +774,6 @@ static int qc_ssl_sess_init(struct quic_conn *qc, SSL_CTX *ssl_ctx, SSL **ssl)
  leave:
 	TRACE_LEAVE(QUIC_EV_CONN_NEW, qc);
 	return ret;
- err:
-	TRACE_DEVEL("leaving on error", QUIC_EV_CONN_NEW, qc);
-	goto leave;
 }
 
 #ifdef HAVE_SSL_0RTT_QUIC
@@ -901,7 +871,18 @@ int qc_alloc_ssl_sock_ctx(struct quic_conn *qc)
 	return !ret;
 
  err:
-	TRACE_DEVEL("leaving on error", QUIC_EV_CONN_NEW, qc);
-	qc_free_ssl_sock_ctx(&ctx);
+	pool_free(pool_head_quic_ssl_sock_ctx, ctx);
 	goto leave;
 }
+
+static void __quic_conn_init(void)
+{
+	ha_quic_meth = BIO_meth_new(0x666, "ha QUIC methods");
+}
+INITCALL0(STG_REGISTER, __quic_conn_init);
+
+static void __quic_conn_deinit(void)
+{
+	BIO_meth_free(ha_quic_meth);
+}
+REGISTER_POST_DEINIT(__quic_conn_deinit);
