@@ -976,9 +976,8 @@ static inline void ssl_ocsp_set_next_update(struct certificate_ocsp *ocsp)
  * the system too much (in theory). Likewise, a minimum 5 minutes interval is
  * defined in order to avoid updating too often responses that have a really
  * short expire time or even no 'Next Update' at all.
- * The ocsp_update_tree lock must be taken by the caller.
  */
-int __ssl_ocsp_update_insert_unlocked(struct certificate_ocsp *ocsp)
+int ssl_ocsp_update_insert(struct certificate_ocsp *ocsp)
 {
 	/* Set next_update based on current time and the various OCSP
 	 * minimum/maximum update times.
@@ -987,23 +986,13 @@ int __ssl_ocsp_update_insert_unlocked(struct certificate_ocsp *ocsp)
 
 	ocsp->fail_count = 0;
 
+	HA_SPIN_LOCK(OCSP_LOCK, &ocsp_tree_lock);
 	ocsp->updating = 0;
 	/* An entry with update_once set to 1 was only supposed to be updated
 	 * once, it does not need to be reinserted into the update tree.
 	 */
 	if (!ocsp->update_once)
 		eb64_insert(&ocsp_update_tree, &ocsp->next_update);
-
-	return 0;
-}
-
-/*
- * Insert a certificate_ocsp structure into the ocsp_update_tree tree.
- */
-int ssl_ocsp_update_insert(struct certificate_ocsp *ocsp)
-{
-	HA_SPIN_LOCK(OCSP_LOCK, &ocsp_tree_lock);
-	__ssl_ocsp_update_insert_unlocked(ocsp);
 	HA_SPIN_UNLOCK(OCSP_LOCK, &ocsp_tree_lock);
 
 	return 0;
@@ -1310,7 +1299,7 @@ static struct task *ssl_ocsp_update_responses(struct task *task, void *context,
 
 		/* if the ocsp_update.http_proxy option was set */
 		if (ocsp_update_dst) {
-			hc->options |= HTTPCLIENT_O_HTTPPROXY;
+			hc->flags |= HC_F_HTTPPROXY;
 			if (!sockaddr_alloc(&hc->dst, ocsp_update_dst, sizeof(*ocsp_update_dst))) {
 				ha_alert("ocsp-update: Failed to allocate sockaddr in %s:%d.\n", __FUNCTION__, __LINE__);
 				goto leave;
@@ -1399,6 +1388,11 @@ http_error:
  */
 static int ssl_ocsp_update_precheck()
 {
+
+	/* the ocsp-update is not usable in the master process */
+	if (master)
+		return ERR_NONE;
+
 	/* initialize the OCSP update dedicated httpclient */
 	httpclient_ocsp_update_px = httpclient_create_proxy("<OCSP-UPDATE>");
 	if (!httpclient_ocsp_update_px)
@@ -1981,7 +1975,7 @@ static int ocsp_update_parse_global_http_proxy(char **args, int section_type, st
 	sockaddr_free(&ocsp_update_dst);
 	/* 'sk' is statically allocated (no need to be freed). */
 	sk = str2sa_range(args[1], NULL, NULL, NULL, NULL, NULL, NULL,
-	                  &errmsg, NULL, NULL,
+	                  &errmsg, NULL, NULL, NULL,
 	                  PA_O_PORT_OK | PA_O_STREAM | PA_O_XPRT | PA_O_CONNECT);
 	if (!sk) {
 		ha_alert("ocsp-update: Failed to parse destination address in %s\n", errmsg);
