@@ -142,44 +142,43 @@ void sedesc_free(struct sedesc *sedesc)
  */
 void se_shutdown(struct sedesc *sedesc, enum se_shut_mode mode)
 {
+	struct sedesc *sdo;
+	struct se_abort_info *reason = NULL;
+	unsigned int flags = 0;
+
+	if ((mode & (SE_SHW_SILENT|SE_SHW_NORMAL)) && !se_fl_test(sedesc, SE_FL_SHW))
+		flags |= (mode & SE_SHW_NORMAL) ? SE_FL_SHWN : SE_FL_SHWS;
+	if ((mode & (SE_SHR_RESET|SE_SHR_DRAIN)) && !se_fl_test(sedesc, SE_FL_SHR))
+		flags |= (mode & SE_SHR_DRAIN) ? SE_FL_SHRD : SE_FL_SHRR;
+
 	if (se_fl_test(sedesc, SE_FL_T_MUX)) {
 		const struct mux_ops *mux = (sedesc->conn ? sedesc->conn->mux : NULL);
-		unsigned int flags = 0;
-
-		if ((mode & (SE_SHW_SILENT|SE_SHW_NORMAL)) && !se_fl_test(sedesc, SE_FL_SHW))
-			flags |= (mode & SE_SHW_NORMAL) ? SE_FL_SHWN : SE_FL_SHWS;
-
-
-		if ((mode & (SE_SHR_RESET|SE_SHR_DRAIN)) && !se_fl_test(sedesc, SE_FL_SHR))
-			flags |= (mode & SE_SHR_DRAIN) ? SE_FL_SHRD : SE_FL_SHRR;
 
 		if (flags) {
 			if (mux && mux->shut) {
-				struct se_abort_info *reason = NULL;
-				struct xref *peer = xref_get_peer_and_lock(&sedesc->xref);
-
-				if (peer) {
-					struct sedesc *sdo = container_of(peer, struct sedesc, xref);
-
+				sdo = se_opposite(sedesc);
+				if (sdo)
 					reason = &sdo->abort_info;
-					xref_unlock(&sedesc->xref, peer);
-				}
-
 				mux->shut(sedesc->sc, mode, reason);
-
 			}
 			se_fl_set(sedesc, flags);
 		}
 	}
 	else if (se_fl_test(sedesc, SE_FL_T_APPLET)) {
-		if ((mode & (SE_SHW_SILENT|SE_SHW_NORMAL)) && !se_fl_test(sedesc, SE_FL_SHW))
-			se_fl_set(sedesc, SE_FL_SHWN);
+		struct appctx *appctx = sedesc->se;
 
-		if ((mode & (SE_SHR_RESET|SE_SHR_DRAIN)) && !se_fl_test(sedesc, SE_FL_SHR))
-			se_fl_set(sedesc, SE_FL_SHRR);
+		if (flags) {
+			if (appctx->applet->shut) {
+				sdo = se_opposite(sedesc);
+				if (sdo)
+					reason = &sdo->abort_info;
+				appctx->applet->shut(appctx, mode, reason);
+			}
+			se_fl_set(sedesc, flags);
+		}
 
 		if (se_fl_test(sedesc, SE_FL_SHR) && se_fl_test(sedesc, SE_FL_SHW))
-			appctx_shut(sedesc->se);
+			appctx_shut(appctx);
 	}
 }
 
@@ -601,7 +600,6 @@ static inline int sc_cond_forward_shut(struct stconn *sc)
 static inline int sc_is_fastfwd_supported(struct stconn *sc)
 {
 	return (!(global.tune.no_zero_copy_fwd & NO_ZERO_COPY_FWD) &&
-		!(sc->flags & SC_FL_NO_FASTFWD) &&
 		sc_ep_test(sc, SE_FL_MAY_FASTFWD_PROD) &&
 		sc_ep_test(sc_opposite(sc), SE_FL_MAY_FASTFWD_CONS) &&
 		sc_ic(sc)->to_forward);
@@ -685,8 +683,7 @@ static void sc_app_shut(struct stconn *sc)
 	}
 
 	sc->flags &= ~SC_FL_NOLINGER;
-	if (!(sc->flags & (SC_FL_EOS|SC_FL_ABRT_DONE)))
-		sc->flags |= SC_FL_ABRT_DONE;
+	sc->flags |= SC_FL_ABRT_DONE;
 	if (sc->flags & SC_FL_ISBACK)
 		__sc_strm(sc)->conn_exp = TICK_ETERNITY;
 
@@ -821,8 +818,7 @@ static void sc_app_shut_conn(struct stconn *sc)
 	}
 
 	sc->flags &= ~SC_FL_NOLINGER;
-	if (!(sc->flags & (SC_FL_EOS|SC_FL_ABRT_DONE)))
-		sc->flags |= SC_FL_ABRT_DONE;
+	sc->flags |= SC_FL_ABRT_DONE;
 	if (sc->flags & SC_FL_ISBACK)
 		__sc_strm(sc)->conn_exp = TICK_ETERNITY;
 }
@@ -1005,8 +1001,7 @@ static void sc_app_shut_applet(struct stconn *sc)
 	}
 
 	sc->flags &= ~SC_FL_NOLINGER;
-	if (!(sc->flags & (SC_FL_EOS|SC_FL_ABRT_DONE)))
-		sc->flags |= SC_FL_ABRT_DONE;
+	sc->flags |= SC_FL_ABRT_DONE;
 	if (sc->flags & SC_FL_ISBACK)
 		__sc_strm(sc)->conn_exp = TICK_ETERNITY;
 }
@@ -1523,6 +1518,10 @@ int sc_conn_recv(struct stconn *sc)
 		sc_conn_eos(sc);
 		ret = 1;
 	}
+	if (sc_ep_test(sc, SE_FL_ERROR)) {
+		sc->flags |= SC_FL_ERROR;
+		ret = 1;
+	}
 
 	/* Ensure sc_conn_process() is called if waiting on handshake. */
 	if (!(conn->flags & (CO_FL_WAIT_XPRT | CO_FL_EARLY_SSL_HS)) &&
@@ -1530,18 +1529,31 @@ int sc_conn_recv(struct stconn *sc)
 		ret = 1;
 	}
 
-	if (sc_ep_test(sc, SE_FL_ERROR)) {
-		sc->flags |= SC_FL_ERROR;
-		ret = 1;
+	if (sc->flags & (SC_FL_EOS|SC_FL_ERROR)) {
+		/* No more data are expected at this stage */
+		se_have_no_more_data(sc->sedesc);
 	}
 	else if (!cur_read &&
 		 !(sc->flags & (SC_FL_WONT_READ|SC_FL_NEED_BUFF|SC_FL_NEED_ROOM)) &&
 		 !(sc->flags & (SC_FL_EOS|SC_FL_ABRT_DONE))) {
-		/* Subscribe to receive events if we're blocking on I/O */
+		/* Subscribe to receive events if we're blocking on I/O. Nothing
+		 * was received and it was not because of a blocking
+		 * condition.
+		 */
 		conn->mux->subscribe(sc, SUB_RETRY_RECV, &sc->wait_event);
 		se_have_no_more_data(sc->sedesc);
 	}
+	else if (sc->flags & SC_FL_EOI) {
+		/* No more data are expected at this stage, except if abortonclose is enabled */
+		if (!(flags & CO_RFL_KEEP_RECV))
+			se_have_no_more_data(sc->sedesc);
+		else
+			se_have_more_data(sc->sedesc);
+	}
 	else {
+		/* The mux may have more data to deliver. Be sure to be able to
+		 * ask it ASAP
+		 */
 		se_have_more_data(sc->sedesc);
 		ret = 1;
 	}
@@ -1784,8 +1796,6 @@ void sc_conn_sync_send(struct stconn *sc)
 		return;
 
 	sc_conn_send(sc);
-	if (oc->flags & CF_WRITE_EVENT)
-		oc->flags |= CF_WAKE_ONCE;
 }
 
 /* Called by I/O handlers after completion.. It propagates
@@ -2287,8 +2297,6 @@ void sc_applet_sync_send(struct stconn *sc)
 		return;
 
 	sc_applet_send(sc);
-	if (oc->flags & CF_WRITE_EVENT)
-		oc->flags |= CF_WAKE_ONCE;
 }
 
 /* Callback to be used by applet handlers upon completion. It updates the stream
